/* ---------------------------------------------------------------- */
//
// StarCatalog.h
//
// Created by Christoph Birk on 2016-06-07
// Copyright (c) 2016 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Foundation/Foundation.h>

#import "Cache.h"

/* ---------------------------------------------------------------- */

#define SCAT_BASE_PATH  "/Users/Shared/StarCatalogs"

/* ---------------------------------------------------------------- */

@interface CatStar : NSObject
{
}

@property (readonly, nonatomic) u_int  num;
@property (readwrite,nonatomic) double alpha,delta,equinox;
@property (readwrite,nonatomic) float  mag,mag_j,mag_h,mag_k;
@property (readonly, nonatomic) char   *spc;

- (instancetype)initWithNum:(u_int)n;
- (BOOL)isEqual:(CatStar*)s;
- (BOOL)isEqual:(CatStar*)s prec:(double)p;

@end

/* ---------------------------------------------------------------- */

@interface StarList : NSObject
{
}

- (instancetype)initWithMaglim:(float)mlim;

@property (readonly, nonatomic) NSMutableArray *array;
@property (readonly, nonatomic) float  maglim;
@property (readwrite,nonatomic) double alpha,delta,equinox; // center

- (CatStar*)getClosestA:(double)a0 D:(double)d0 E:(double)e0;

@end

/* ---------------------------------------------------------------- */

enum catalog_index_enum {
     SCAT_SAO,SCAT_DIMM,
     SCAT_TYCHO,
     SCAT_GSC,SCAT_GSCPLUS,
     SCAT_USNO,SCAT_USNO13,SCAT_USNO14,SCAT_USNO15,SCAT_USNO16,
     SCAT_MASS,
     SCAT_GAIA19,SCAT_GAIA17,SCAT_GAIA15,SCAT_GAIA13,
     SCAT_N
};

@interface StarCatalog : NSObject <Cache_Delegate>

@property (readonly,nonatomic) int    catalog;
@property (readonly,nonatomic) double equinox;
@property (readonly,nonatomic) float  mag;

+ (int)indexFromName:(NSString*)name;
+ (NSString*)nameFromIndex:(int)i;

- (BOOL)loadCatalog:(int)c path:(NSString*)folder;
- (BOOL)loadCatalog:(NSString*)name;
- (StarList*)getList:(double)a D:(double)d E:(double)e W:(float)w H:(float)h M:(float)m;
- (NSString*)starName:(u_int)number;
- (NSString*)catName;
- (NSString*)starTag:(CatStar*)star;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
