/* ---------------------------------------------------------------- */
//
// TextField.m
//
// Created by Christoph Birk on 2013-01-22
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG           0              // debug level

/* ---------------------------------------------------------------- */

#import "TextField.h"

/* ---------------------------------------------------------------- */

@implementation TextField

@synthesize isEditing;

/* ---------------------------------------------------------------- */

#if 0 // not implemented
- (instancetype)initWithCoder:(NSCoder*)coder
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif

  return [super initWithCoder:coder];  
}
#endif

// An awakeFromNib message is sent to each object loaded from the 
// archive, but only if it can respond to the message, and only 
// after all the objects in the archive have been loaded and initialized.
// When an object receives an awakeFromNib message, it is guaranteed 
// to have all its outlet instance variables set.

#if 0 // not implemented
- (void)awakeFromNib
{
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif

  if ([[super class] instancesRespondToSelector:@selector(awakeFromNib)]) {
    [super awakeFromNib];
  }
}
#endif

/* ---------------------------------------------------------------- */

-(void)cancelOperation:(id)sender
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);
  
  if (currentValue) [self setStringValue:currentValue];
  [self setTextColor:[NSColor blackColor]];
  isEditing = FALSE;
}

/* ---------------------------------------------------------------- */

- (void)storeValue:(char*)text
{
#if (DEBUG > 1)
  fprintf(stderr,"%s()\n",PREFUN);
#endif
  assert([NSThread isMainThread]);

  [self setStringValue:[NSString stringWithCString:text encoding:NSASCIIStringEncoding]];
  if (currentValue) [currentValue release];
  currentValue = [[NSString alloc]  initWithString:[self stringValue]];
  [self setTextColor:[NSColor blackColor]];
  isEditing = FALSE;
}

/* ---------------------------------------------------------------- */

+ (BOOL)isValidNumberString:(NSString*)string
{
  unichar c; 
  int     i,np=0;

  if ([string length] == 0) return FALSE;

  assert([NSThread isMainThread]);
  
  for (i=0; i<[string length]; i++) {
    c = [string characterAtIndex:i];
    if (isdigit(c)) {
      continue;
    } else if (c == '+') {
      if (i == 0) continue;
    } else if (c == '-') {
      if (i == 0) continue;
    } else if (c == '.') { 
      np++;
      if (np < 2) continue;
    } else if (c == '?') {
      if (1+i == [string length]) continue;
    }
    return FALSE;
  }

  return TRUE;
}

/* ---------------------------------------------------------------- */

- (BOOL)hasValidNumberString
{
  return [TextField isValidNumberString:[self stringValue]];
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
