/* ---------------------------------------------------------------- */
//
// AppDelegate.h
// Henrietta
//
// Created by Christoph Birk on 2022-06-05
// Copyright © 2022 Carnegie Observatories. All rights reserved.
//
// TODO ?Nick ?Greg ?Francesco
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

#import "CameraController.h"
#import "TempController.h"
#import "GalilHardhat.h"
#import "Logger.h"

#import "main.h"

/* ---------------------------------------------------------------- */

@interface AppDelegate : NSObject <NSApplicationDelegate>
{
  // GUI - configuration
  IBOutlet NSTextField   *edit_observer;
  IBOutlet NSTextField   *edit_archon,*edit_lake,*edit_galil,*edit_tcs;
  IBOutlet NSPopUpButton *popup_archon,*popup_lake,*popup_galil,*popup_tcs;
  IBOutlet NSButton      *chk_archon,*chk_lake,*chk_tcpip;
  // GUI - Preferences
  IBOutlet NSPanel       *panel_prefs;
  IBOutlet NSButton      *chk_eng;
  IBOutlet NSTextField   *edit_snd_expdone;
  IBOutlet NSTextField   *edit_satlev,*edit_email;
  IBOutlet NSTextField   *edit_thinning,*label_thinning;
  IBOutlet NSButton      *but_snd_expdone;
  IBOutlet NSButton      *chk_disperser;
  IBOutlet NSTextField   *edit_heater; // heater low warning TODO ?Nick 
  IBOutlet NSButton      *chk_showRaw,*chk_saveRaw;
  IBOutlet NSButton      *chk_showFrames,*chk_saveFrames;
  IBOutlet NSButton      *chk_showCurrent;  // b0091
  IBOutlet NSPopUpButton *popup_refPixel;
  // GUI - messages
  IBOutlet NSPanel       *panel_log;
  IBOutlet NSTableView   *messageTable;
  // GUI - data path
  IBOutlet NSPanel       *panel_datapath;
  IBOutlet NSTextField   *edit_path1,*edit_path2;
}

@property (readonly,nonatomic) CameraController *camController;
@property (readonly,nonatomic) TempController *tempController;
@property (readonly,nonatomic) GalilHardhat *galilHH;
@property (readonly,nonatomic) float heaterWarning; // ?Nick
@property (readonly,nonatomic) int engMode,disperser,thinTime;

+ (NSString*)errorString:(int)err;
+ (NSString*)makeExcuse;
+ (void)goodbye:(NSString*)text error:(int)err;

- (IBAction)action_startup:(id)sender;
- (IBAction)action_file:(id)sender;
- (IBAction)action_prefs:(id)sender;
- (IBAction)action_datapath:(id)sender;
- (IBAction)action_camera:(id)sender;
- (IBAction)action_help:(id)sender;

- (NSArray*)dataPaths;
- (void)emailText:(const char*)text;
- (void)message:(NSString*)text level:(int)lev time:(int)dt;
- (void)message:(NSString*)text level:(int)lev;
- (void)alert:(const char*)text level:(int)lev dt:(int)dt;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

