/* ---------------------------------------------------------------- */
//
// ArchonController.h
// PFS
//
// Created by Christoph Birk on 2015-07-30
// Copyright (c) 2015 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Foundation/Foundation.h>

#import "ArchonIO.h"

/* ---------------------------------------------------------------- */

#define DBE_ARCHON_ONLINE  "dbe_archon_online"
#define DBE_ARCHON_HOST    "dbe_archon_host"
#ifdef SIM_ONLY
#define DEF_ARCHON_HOST     "localhost"
#else
#define DEF_ARCHON_HOST    "10.7.45.75"  
#endif
#define H2RG_FRAME_TIME    (1.352)     // 4CHAN:(8*) TODO check latest .acf

enum sta_errors_enum { E_sta_error=E_archon_last+1,
                       E_sta_power,
                       E_sta_config,
                       E_sta_parameter,
                       E_sta_last
};

/* ---------------------------------------------------------------- */

@interface ArchonController : NSObject <NSTableViewDataSource> 
{
  // STA Hardhat panel 
  IBOutlet NSPanel *panel;
  IBOutlet NSTableView *responseTable;
  NSMutableArray *respArray;
  int      resp_n,resp_head;
  IBOutlet NSTextField *edit_p2v5,*edit_p5v,*edit_p6v,*edit_n6v; 
  IBOutlet NSTextField *edit_p17v,*edit_n17v,*edit_p35v;
  IBOutlet NSTextField *edit_tmbp;
  IBOutlet NSTextField *edit_tm01,*edit_tm02,*edit_tm03;
  IBOutlet NSTextField *edit_tm06,*edit_tm07,*edit_tm11;
  IBOutlet NSTextField *edit_acf;
  // STA bias panel b0083
  IBOutlet NSPanel *panel_biases;
  NSTextField *label_lvlc[LVLC_N],*edit_lvlcP[LVLC_N],*edit_lvlcV[LVLC_N],*edit_lvlcI[LVLC_N];
  NSTextField *label_lvhc[LVHC_N],*edit_lvhcP[LVHC_N],*edit_lvhcV[LVHC_N],*edit_lvhcI[LVHC_N];
}

@property (readonly,nonatomic)  u_int  ntaps;       // array geometry
@property (readonly,nonatomic)  u_int  dimx,dimy;
@property (readonly,atomic)     BOOL   configured,array_power; 
@property (readonly,atomic)     long   last_check,interval;
@property (readonly,nonatomic)  int    online;

@property (readwrite,nonatomic) float  exptime;
@property (readwrite,nonatomic) int    shutter,objects;  // simulator mode b0091

@property (readonly,atomic)     BOOL   aborted,fetching; 

@property (readonly,nonatomic)  u_int  dataPixels,dataLines;

@property (readonly,nonatomic)  double start_int;   // epoch 1970
@property (readonly,nonatomic)  double egain,enoise;

@property (readwrite,nonatomic,copy) NSString *acf_file,*acf_version;

+ (NSString*)errorString:(int)err;

- (IBAction)action_status:(id)sender;
- (IBAction)action_frames:(id)sender;
- (IBAction)action_recon:(id)sender;
- (IBAction)action_poweroff:(id)sender;
- (IBAction)action_biases:(id)sender;
- (IBAction)action_terminal:(id)sender;
- (IBAction)action_acf:(id)sender;

- (int)startExposure:(int)nf;
- (int)startExposure:(int)nf dummies:(int)nd;
- (int)abortExposure;
- (NSData*)fetchFrame:(float)et;

- (void)setupSimulator:(float)ps;
- (void)setA:(double)a D:(double)d E:(double)e S:(int)s;

- (int)statusCheck:(BOOL)reset;

- (void)printBufferState:(double)dt; 
- (int)lockNextFrame:(double)timeout;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
