/* ---------------------------------------------------------------- */
//
// ArchonIO.h
// PFS
//
// Created by Christoph Birk on 2015-07-29
// Copyright (c) 2015 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import "Tcpip.h"

/* ---------------------------------------------------------------- */

#undef  LOG_PSU

enum archon_errors_enum { E_archon_error=E_tcpip_error+400, 
                          E_archon_response,
                          E_archon_response_id,
                          E_archon_response_lf,
                          E_archon_command,
                          E_archon_status,
                          E_archon_parameter,
                          E_archon_last
};

#define ARCHON_NBUFS      3
#define ARCHON_TIMER_TIC  10.0e-9        // 10 nano-seconds
#define ARCHON_NULL       "(null)"

typedef struct {
  u_int  base;
  int    width,height,lines,frame;
  BOOL   complete;
  u_long timestamp;
  double deltatime;
} ArchonFrame;

#define LVLC_N          13    // low current biases b0089 (HyVisi:13,IR:12)
#define LVHC_N          6     // number of high current biases

/* ---------------------------------------------------------------- */

@interface ArchonIO : TCPIP_Connection
{
  short  messageID;
  double uptime;
  int    wbuf,rbuf;
  ArchonFrame *frames;
}

@property (readwrite,atomic)   int    systemChange;
@property (readonly)           int    power,nlogs;
@property (readonly)           double uptime;
@property (readonly)      ArchonFrame *frames;
@property (readonly)           int    wbuf,rbuf;
@property (readonly,nonatomic) float  temp_backplane;
@property (readonly,nonatomic) float  p2v5_v,p5v_v,p6v_v,n6v_v,p17v_v,n17v_v,p35v_v;
#ifdef LOG_PSU
@property (readonly,nonatomic) float  p2v5_i,p5v_i,p6v_i,n6v_i,p17v_i,n17v_i,p35v_i;
#endif
@property (readonly,nonatomic) float  temp_mod1,temp_mod2,temp_mod3;
@property (readonly,nonatomic) float  temp_mod6,temp_mod7,temp_mod11;
@property (readonly,nonatomic) float  *lvlc_v,*lvlc_i,*lvhc_v,*lvhc_i; // b0082

@property (readonly,nonatomic,copy) NSString *back_version;
@property (readonly,atomic,copy)    NSString *response;

- (int)request:(const char*)cmd response:(char*)res;
- (int)command:(const char*)cmd;
- (int)reopen:(int)delay;

- (int)getSystem;
- (int)getStatus;
- (NSArray*)getLog;
- (int)updateFrames;
- (int)updateFrame:(int)b;
- (int)getTimer;

- (int)uploadConfiguration:(NSArray*)config;
- (int)setParameterFast:(const char*)par to:(int)value;
- (u_short*)fetchBytes:(u_int)nbytes fromBuffer:(int)b offset:(u_int)offset;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
