/* ---------------------------------------------------------------- */
//
// CameraController.h
// Henrietta
//
// Created by Christoph Birk on 2022-09-02
// Copyright (c) 2022 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

#import "ArchonController.h"
#import "QltoolController.h"

/* ---------------------------------------------------------------- */

#define N_FOLDERS       2
#define DBE_DATAPATH1   "dbe_datapath1" 
#define DBE_DATAPATH2   "dbe_datapath2"

enum camera_errors_enum { E_camera_error=2000,
                          E_camera_timeout,
                          E_camera_lines,
                          E_camera_abort,
                          E_camera_sync,
                          E_camera_fits,
                          E_camera_last
};


#define DBE_SND_EXPDONE "dbe_snd_expdone"
#define DEF_SND_EXPDONE "'exposure complete'"

#define DBE_OBSERVER    "dbe_observer"
#define DEF_OBSERVER    "N.N."

#define DBE_REFPIXMODE  "dbe_refPixelMode" 
#define DBE_SHOWRAW     "dbe_showRaw"
#define DBE_SAVERAW     "dbe_saveRaw"
#define DBE_SHOWFRAMES  "dbe_showFrames"
#define DBE_SAVEFRAMES  "dbe_saveFrames"
#define DBE_SHOWCURRENT "dbe_showCurrent"

/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

@interface CameraController : NSWindowController
{
  volatile BOOL loop_doing; 
  int     loop,nloops,run_number;
  int     exptype,readmode;
  char    etStr[32],rmStr[32];
  char    object[64],comment[64]; 
  float   pixscale;
  BOOL    warningPosted[N_FOLDERS],criticalPosted[N_FOLDERS];
  char    snd_expdone[512]; 
  char    observer[64];
  time_t  next_disk;
  NSMutableArray *datapaths;
  // GUI
  IBOutlet NSTextField *edit_exptime,*edit_runtime;
  IBOutlet NSTextField *edit_nloops,*edit_loop;
  IBOutlet NSTextField *label_readmode; // b0092
  IBOutlet NSProgressIndicator *prog_exposure; 
  IBOutlet NSPopUpButton *popup_exptype,*popup_readmode;
  IBOutlet NSPopUpButton *drop_slit,*drop_filter,*drop_diffuser,*drop_grism;
  IBOutlet NSPopUpButton *drop_slide;
           NSArray *drop_wheels;
  IBOutlet NSButton *but_start,*but_stop;
  IBOutlet NSTextField *edit_object,*edit_comment,*edit_file;
  IBOutlet NSLevelIndicator *level_disk1,*level_disk2;
           NSArray          *level_disks;
  // QlTool
  QltoolController *qltool;
  // Hardhat - STA (Archon)
  IBOutlet ArchonController *staController;
  IBOutlet NSPanel *panel_sta;
  // Telescope
  double alpha,delta,equinox;
}

@property (readonly,nonatomic)  NSPanel *panel_sta;
@property (readonly,atomic)     BOOL    loop_doing;

@property (readonly,nonatomic) int refPixelMode;
@property (readonly,nonatomic) BOOL saveRaw,showRaw,saveFrames,showFrames,showCurrent;
@property (readwrite,atomic,copy) NSString *tcpResponse;

+ (NSString*)errorString:(int)err;

- (void)reloadPreferences;
- (void)reloadDatapath;
- (void)showQltool;
- (void)loadFITS:(NSURL*)url asSky:(BOOL)sky;
- (int)updateWheel:(int)w fromGalil:(BOOL)refresh;
- (int)motorMoving;

- (IBAction)action_file:(id)sender;

- (IBAction)action_exptime:(id)sender;
- (IBAction)action_exptype:(id)sender;
- (IBAction)action_nloops:(id)sender;
- (IBAction)action_readmode:(id)sender;
- (IBAction)action_motor:(id)sender;
- (IBAction)action_start:(id)sender;
- (IBAction)action_stop:(id)sender;

@end

/* ---------------------------------------------------------------- */
