/* ---------------------------------------------------------------- */
//
// DataSimulator.h
// PFS
//
// Created by Christoph Birk on 2016-06-07
// Copyright (c) 2016 Christoph Birk. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Foundation/Foundation.h>

#import "MTwister.h"
#import "StarCatalog.h"

/* ---------------------------------------------------------------- */

@interface FovStar : NSObject
{
}

@property (readonly,nonatomic) float xc,yc,mag;

+ (FovStar*)starAtX:(float)x Y:(float)y M:(float)m;
- (FovStar*)initAtX:(float)x Y:(float)y M:(float)m;

@end

/* ---------------------------------------------------------------- */

@interface ArraySimulator : NSObject
{
  NSString *name;
  u_short  *data0;
  int      width,height,read_lines;
  BOOL     flip_x,flip_y; 
  float    choffx,choffy;
  MTwister *twister;
}

- (id)initW:(int)w H:(int)h X:(float)x Y:(float)y;

- (void)reset;
- (void)update:(float)cur_exptime;
- (void)readout:(double)phase;

@property (readonly,nonatomic) NSString *name;
@property (readwrite,nonatomic) int    bias_level,disperser;
@property (readwrite,nonatomic) int    transpose,gain; // todo not really used
@property (readwrite,nonatomic) int    objects,shutter; // { stars,background,dark}
@property (readwrite,nonatomic) double efzm,fwhm,back,blur;
@property (readwrite,nonatomic) double egain,enoise;
@property (readwrite,nonatomic) double pixscale,neutral_filter;
@property (readwrite,nonatomic) double alpha,delta,equinox;
@property (readwrite,nonatomic,retain) StarCatalog *scat;
@property (readonly,nonatomic) u_short *data0;
@property (readonly,nonatomic) int     dimx,dimy;
@property (readwrite,nonatomic) BOOL   leftRef,rightRef;
@property (readwrite,nonatomic) double drift_a,drift_f;

@end

/* ---------------------------------------------------------------- */

@interface DataSimulator : NSObject
{
  NSMutableArray *arrays;
  StarCatalog *scat;
}

@property (readwrite,nonatomic) int    disperser,shutter,objects;
@property (readwrite,nonatomic) BOOL   transposed;
//@property (readonly,atomic)     BOOL   done;

- (id)initWithPixscale:(float)ps;
- (void)addArrayW:(int)w H:(int)h X:(float)x Y:(float)y L:(BOOL)lr R:(BOOL)rr;
- (void)setEgain:(double)eg Enoise:(double)en;
- (void)setAlpha:(double)alp delta:(double)del equinox:(double)equ;
- (void)startExposure;
- (void)updateDetector:(float)et;
- (NSData*)fetchFrame;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
