/* ---------------------------------------------------------------- */
//
// Galil.h
// TeleGUI/Gmap/Swope
//
// Created by Christoph Birk on 2013-07-17
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

#import "Tcpip.h"
#import "Logger.h"
#import "main.h"                      // SimulatorOnly flag

/* ---------------------------------------------------------------- */

enum galil_errors_enum { E_galil_error=E_tcpip_error+300,
                         E_galil_command,
                         E_galil_fnotf,
                         E_galil_forlim,
                         E_galil_revlim,
                         E_galil_stop,
                         E_galil_timeout,
                         E_galil_pnotf,
                         E_galil_nohome,
                         E_galil_abort,
                         E_galil_nodetent,
                         E_galil_target,
                         E_galil_last
};

#define GALIL_WHEELS         4
// unused #define GALIL_GUI_PORT       2000
#define GALIL_HARD_PORT      2001

#define GALIL_RESPONSE_SIZE  512

extern Logger *galil_logger;

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

@interface GalilConnection : TCPIP_Connection {
}

- (int)request:(const char*)cmd response:(char*)res;
- (int)command:(const char*)cmd;

- (int)errorMessage:(char*)string;

- (int)set_var:(const char*)key withInt:(int)value;
- (int)get_var:(const char*)key intPtr:(int*)value;
- (int)get_input:(int)bit intPtr:(int*)value;
- (int)get_output:(int)bit intPtr:(int*)value;
- (int)set_output:(int)bit value:(int)value;

- (int)setupFromFile:(const char*)file;

@property (readonly,nonatomic) BOOL fileUploaded;

@end

/* ---------------------------------------------------------------- */

@interface GalilObject : NSObject {
  GalilConnection *galil;
  NSString        *name;
  int             state;
};

@property (readonly)        GalilConnection *galil;
@property (readonly)        int      state;
@property (readonly,retain) NSString *name;

+ (NSString*)errorString:(int)err;

- (id)initWithConnection:(GalilConnection*)controller;
- (int)setup:(const char*)base;

- (int)move:(int)pos;
- (int)get;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

@interface GalilAxis : GalilObject {   // Note: Axis has no 'state'
  char  axis;
  int   speed,acc,dec;
  int   limfor,limrev;
  BOOL  motor_off,has_limits;
}

@property (readonly,nonatomic) char  axis;
@property (readonly,nonatomic) int   run_speed,acc,dec,gain;
@property (readonly,nonatomic) BOOL  motor_off;
@property (readonly)           int   speed,jog_speed,step_pos;
@property (readonly,nonatomic) int   spt;
@property (readonly,nonatomic) float velocity;
@property (readonly)           int   limfor,limrev;
@property (readonly)           BOOL  running,jogging;
@property (readonly,nonatomic) int   stop_code;
@property (readonly,nonatomic) BOOL  fileUploaded;

- (id)initWithConnection:(GalilConnection*)controller axis:(char)a;
- (int)setupFromFile:(const char*)file;

- (int)set_speed:(int)val;
- (int)get_speed;
- (int)get_velocity;
- (int)set_acceleration:(int)val;
- (int)get_acceleration;
- (int)set_deceleration:(int)val;
- (int)get_deceleration;
- (int)set_step_pos:(int)val;
- (int)get_step_pos;
- (int)get_limits;
- (int)get_pospar;
- (int)get_spdpar;

- (int)moveSteps:(int)steps wait:(BOOL)wait;
- (int)motorPower:(BOOL)power;
- (int)stop;
- (int)jog:(int)spd;
- (BOOL)isRunning;
- (BOOL)hasStopped;
- (BOOL)isJogging;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

@interface GalilSlide : GalilObject { 
};

- (int)setup:(const char*)base fromPath:(const char*)path withAxis:(GalilAxis*)a;

@property (readonly) NSString *slideName;
@property (readonly) GalilAxis *axis;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

@interface GalilWheel : GalilObject {
}

@property (readonly) NSString *wheelName;
@property (readonly) GalilAxis *axis;
@property (readonly) int  detent,homesw;
@property (readonly) BOOL foundHome,homing;
@property (readonly) int  nPositions,stepsPerPos;
@property (readonly) NSArray *positionNames;

- (int)setup:(const char*)base fromPath:(const char*)path withAxis:(GalilAxis*)a;
- (int)get_switches;
- (int)find_home;
- (int)find_detent;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
