/* ---------------------------------------------------------------- */
//
// GalilHardhat.m
// Swope / Henrietta
//
// Created by Christoph Birk on 2013-07-19
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG           1

//#define UNIT_TEST       0
#define TIME_TEST       0

/* ---------------------------------------------------------------- */

#import  "GalilHardhat.h"
#import  "AppDelegate.h"
#import  "Notice.h"
#import  "Yutils.h"
#import  "main.h"                      // SimulatorOnly flag
#import  "CameraController.h"

#include "utils.h"

/* ---------------------------------------------------------------- */

#define GALIL_AXES           5
#define GALIL_WHEELS         4

static int active[GALIL_AXES] = {1,1,1,1,1}; // testing only
static int wheels[GALIL_WHEELS] = {1,1,1,1}; // testing only

#define FRAME_GALIL_HARDHAT  "GalilHardhat"
#define DBE_GALIL_ID         "dbe_galil_id"

/* ---------------------------------------------------------------- */

@implementation GalilHardhat
{
  AppDelegate *app;
  GalilConnection *gConnection; 
  GalilAxis       *gAxis[GALIL_AXES]; // TODO replace with NSArray ?
  GalilWheel      *gWheel[GALIL_WHEELS];
  NSTimer *theTimer;
  u_int   counter;
  NSControlStateValue check_states[GALIL_AXES];
}

/* ---------------------------------------------------------------- */
#pragma mark "Class"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

+ (NSString*)errorString:(int)err
{
  NSString *string=nil;
  
  switch (err) {
  default:
    if (!string) string = [GalilObject errorString:err];
    break;
  }
  if (!string) {
    string = [NSString stringWithFormat:@"GalilHardhat: %@ (%d)",
                       [AppDelegate makeExcuse],err];
  }
  
  return string;
}

/* ---------------------------------------------------------------- */
#pragma mark "NSWindowController"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (id)initWithWindowNibName:(NSString*)name
{
  int  i;
  char host[256];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,name);
#endif

  if (!(self = [super initWithWindowNibName:name])) return nil;
  
  app = [NSApp delegate];        // callback when closed

#if defined(SIM_ONLY) && defined(NDEBUG)
  strcpy(host,LOCALHOST);        // enforce simulator
#else
  YprefGetChar(DBE_GALIL_HOST,DEF_GALIL_HOST,host,sizeof(host),use_shared);
#endif
  online = YprefGetInt(DBE_GALIL_ONLINE,DEF_STATE,use_shared);
  gConnection = [[GalilConnection alloc] initWithHost:host
                                         port:GALIL_HARD_PORT
                                         online:online];
  for (i=0; i<GALIL_AXES; i++) {
    gAxis[i] = [[GalilAxis alloc] initWithConnection:gConnection axis:('A'+i)];
    check_states[i] = NSControlStateValueOff;
  }
  jog_lock = [[NSLock alloc] init];

  for (i=0; i<GALIL_WHEELS; i++) {
   gWheel[i] = [[GalilWheel alloc] initWithConnection:gConnection];
  }
  
  command_history = [[NSMutableArray alloc] initWithCapacity:32];
  
  [[self window] setFrameUsingName:@FRAME_GALIL_HARDHAT];
  [self performSelectorInBackground:@selector(run_setup) withObject:nil];

#ifndef NDEBUG
  assert([NSThread isMainThread]);
  theTimer = [NSTimer scheduledTimerWithTimeInterval:0.5
                      target:self selector:@selector(fireTimer:)
                      userInfo:nil repeats:YES];
#endif

  return self;
}

/* ---------------------------------------------------------------- */

- (void)awakeFromNib
{
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif
  
  check_axis = [@[check_A,check_B,check_C,check_D,check_E] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:check_axis]; assert([set count]==GALIL_AXES); }
#endif

  edit_stp = [@[edit_stpA,edit_stpB,edit_stpC,edit_stpD,edit_stpE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_stp]; assert([set count]==GALIL_AXES); }
#endif

  edit_rel = [@[edit_relA,edit_relB,edit_relC,edit_relD,edit_relE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_rel]; assert([set count]==GALIL_AXES); }
#endif

  edit_spd = [@[edit_spdA,edit_spdB,edit_spdC,edit_spdD,edit_spdE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_spd]; assert([set count]==GALIL_AXES); }
#endif

  edit_acc = [@[edit_accA,edit_accB,edit_accC,edit_accD,edit_accE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_acc]; assert([set count]==GALIL_AXES); }
#endif
  
  edit_dec = [@[edit_decA,edit_decB,edit_decC,edit_decD,edit_decE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_dec]; assert([set count]==GALIL_AXES); }
#endif

  edit_vel = [@[edit_velA,edit_velB,edit_velC,edit_velD,edit_velE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:edit_vel]; assert([set count]==GALIL_AXES); }
#endif

  but_relp = [@[but_relpA,but_relpB,but_relpC,but_relpD,but_relpE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_relp]; assert([set count]==GALIL_AXES); }
#endif

  but_relm = [@[but_relmA,but_relmB,but_relmC,but_relmD,but_relmE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_relm]; assert([set count]==GALIL_AXES); }
#endif

  but_stop = [@[but_stopA,but_stopB,but_stopC,but_stopD,but_stopE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_stop]; assert([set count]==GALIL_AXES); }
#endif

  but_clr = [@[but_clrA,but_clrB,but_clrC,but_clrD,but_clrE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_clr]; assert([set count]==GALIL_AXES); }
#endif

  but_limf = [@[but_limfA,but_limfB,but_limfC,but_limfD,but_limfE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_limf]; assert([set count]==GALIL_AXES); }
#endif
  but_limr = [@[but_limrA,but_limrB,but_limrC,but_limrD,but_limrE] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_limr]; assert([set count]==GALIL_AXES); }
#endif

  slider_jog = [@[slider_jogA,slider_jogB,slider_jogC,slider_jogD,slider_jogE] retain];
#ifndef NDEBUG
  { NSSet *set=[NSSet setWithArray:slider_jog]; assert([set count]>=GALIL_AXES); }
#endif

  drop_wheel = [@[drop_wheel1,drop_wheel2,drop_wheel3,drop_wheel4] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:drop_wheel]; assert([set count]==GALIL_WHEELS); }
#endif
  but_dets = [@[but_dets1,but_dets2,but_dets3,but_dets4] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_dets]; assert([set count]==GALIL_WHEELS); }
#endif
  but_homs = [@[but_homs1,but_homs2,but_homs3,but_homs4] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_homs]; assert([set count]==GALIL_WHEELS); }
#endif
  but_home = [@[but_home1,but_home2,but_home3,but_home4] retain];
#ifndef NDEBUG
  { NSSet *set = [NSSet setWithArray:but_home]; assert([set count]==GALIL_WHEELS); }
#endif

  [galil_logger setTable:messageTable];
  
  // Note: without formatter Mavericks displays '1,000' 
  NSNumberFormatter *formatter = [[[NSNumberFormatter alloc] init] autorelease];
  for (int i=0; i<GALIL_AXES; i++) {   // disable unused axes
    [edit_stp[i] setFormatter:formatter];
    [edit_spd[i] setFormatter:formatter];
    [edit_acc[i] setFormatter:formatter];
    [edit_dec[i] setFormatter:formatter];
    [edit_vel[i] setFormatter:formatter];
    [check_axis[i] setEnabled:(active[i] ? YES : NO)];
    [edit_spd[i] setEnabled:(active[i] ? YES : NO)];
    [edit_acc[i] setEnabled:(active[i] ? YES : NO)];
    [edit_dec[i] setEnabled:(active[i] ? YES : NO)];
    [but_stop[i] setEnabled:(active[i] ? YES : NO)];
    [but_clr[i] setEnabled:(active[i] ? YES : NO)];
  }
}

/* ---------------------------------------------------------------- */

- (BOOL)windowShouldClose:(id)sender   // catch 'close' (red) button
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p): sender=%p\n",PREFUN,self.window,sender);
#endif

  [self.window saveFrameUsingName:@FRAME_GALIL_HARDHAT]; 
  [self.window orderOut:self];         // no closing -- just hide
  return NO;
}

/* --- */

#if 0 // not used anymore -- we never close, just hide the window
- (void)windowWillClose:(NSNotification*)note
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p): window=%p\n",PREFUN,note,[note object]);
#endif
  assert([note object] == [self window]);
  [theTimer invalidate];               // released by NSRunLoop
  [self.window saveFrameUsingName:@FRAME_GALIL_HARDHAT];
}
#endif

/* ---------------------------------------------------------------- */

- (void)dealloc
{
  int i;
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif
  assert([NSThread isMainThread]);
  
  [galil_logger setTable:nil];

  for (i=0; i<GALIL_AXES; i++) [gAxis[i] release]; // "release the hounds"
  [jog_lock release];
  [command_history release];           // release arrays
  [check_axis release];
  [edit_stp release];
  [edit_rel release];
  [edit_spd release];
  [edit_acc release];
  [edit_dec release];
  [edit_vel release];
  [but_relp release];
  [but_relm release];
  [but_stop release];
  [but_clr release];
  [but_limf release];
  [but_limr release];
  [slider_jog release];

  for (i=0; i<GALIL_WHEELS; i++) [gWheel[i] release];
  [drop_wheel release];
  [but_dets release];
  [but_homs release];
  [but_home release];

  [gConnection release];
  
  [super dealloc];
}

/* ---------------------------------------------------------------- */
#pragma mark "Instance"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

#ifndef NDEBUG
- (void)fireTimer:(NSTimer*)timer
{  
  counter++;
  if (!_configured) return;
}
#endif

/* ---------------------------------------------------------------- */

- (BOOL)check_configured
{
  if (!_configured) {
    [Notice openNotice:"Galil not configured" title:p_exec time:15 level:1];
    return FALSE;
  }
  return TRUE;
}

/* ---------------------------------------------------------------- */

- (void)showError:(int)err name:(const char*)name
{
  char buf[256];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d,%s)\n",PREFUN,err,name);
#endif

  if (err == E_galil_stop) return;     // ignore 'ST' command
  
  sprintf(buf,"%s\rfailed, err=%d",name,err);
  if (err == E_galil_command) { char errstr[128];
    err = [gConnection errorMessage:errstr];
    if (!err) { strcat(buf,"\r"); strcat(buf,errstr); }
  } else
  if (err == E_galil_nohome) {
    strcat(buf,"\rHOME not found");
  }
  [Notice openNotice:buf title:p_exec time:60 level:1];
}

/* ---------------------------------------------------------------- */

- (void)updateWheelFromAxis:(int)a
{
   for (int w=0; w<GALIL_WHEELS; w++) { // find appropriate wheel
     if (gAxis[a].axis == gWheel[w].axis.axis) {
       if (!gConnection.online) [gWheel[w].axis set_step_pos:gAxis[a].step_pos];
       [self updateWheel:w fromGalil:NO]; // we use the same GalilAxis TEST@SBS
       break;
     }
  }
}

/* --- */

- (void)updateAxisFromWheel:(int)w
{
  for (int a=0; a<GALIL_AXES; a++) { // find appropriate axis
     if (gAxis[a].axis == gWheel[w].axis.axis) {
       if (!gConnection.online) [gAxis[a] set_step_pos:gWheel[w].axis.step_pos];
       [self updateAxis:a fromGalil:NO]; // we use the same GalilAxis TEST@SBS
       break;
     }
  }
}

/* ---------------------------------------------------------------- */
#pragma mark "Actions"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (IBAction)action_check:(id)sender
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);
  assert([sender isKindOfClass:[NSButton class]]);

  NSControlStateValue state = [(NSButton*)sender state];
  NSInteger i = [check_axis indexOfObject:sender];
  assert(i != NSNotFound);
  check_states[i] = state;

  if (![self check_configured]) return;

  if (state == NSControlStateValueOn) {
    if (!check_open) {
      [self performSelectorInBackground:@selector(run_check) withObject:nil];
    }
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_clear:(id)sender
{
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  if (![self check_configured]) return;

  if ((i=[but_clr indexOfObject:sender]) == NSNotFound) {
    assert(0);
    return;
  }
  int err = [gConnection open];
  if (!err) {
    if (!err) err = [gAxis[i] set_step_pos:0];
    [gConnection close];
  }
  if (err) { char buf[128];
    sprintf(buf,"%s\rfailed, err=%d",PREFUN,err);
    [Notice openNotice:buf title:p_exec time:30 level:1];
  }
  [self updateAxis:(int)i fromGalil:NO];
}

/* ---------------------------------------------------------------- */

- (IBAction)action_relative:(id)sender
{
  int steps=0;
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  if (![self check_configured]) return;

  if ((i=[but_relp indexOfObject:sender]) != NSNotFound) {
    steps =  [edit_rel[i] intValue];
  } else 
  if ((i=[but_relm indexOfObject:sender]) != NSNotFound) {
    steps = -[edit_rel[i] intValue];
  } else {
    assert(0);
  }
  if (steps != 0) {                    // move axis in thread  
    [self enable:NO axis:(int)i];
    [self performSelectorInBackground:@selector(run_waitForAxis:) 
          withObject:[NSNumber numberWithInteger:i]];
    dispatch_async(GLOBAL_QUEUE,^(void) {  // default priority
      int err = [gAxis[i] moveSteps:steps wait:YES];
      if (err) [self showError:err name:PREFUN];
      [self updateAxis:(int)i fromGalil:YES];
      [self enable:YES axis:(int)i];
      [self updateWheelFromAxis:(int)i]; 
    });
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_edit:(id)sender
{
  int err=0;
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  if (![self check_configured]) return;

  if ((i=[edit_spd indexOfObject:sender]) != NSNotFound) {
    err = [gAxis[i] set_speed:[sender intValue]];
  } else
  if ((i=[edit_acc indexOfObject:sender]) != NSNotFound) {
    err = [gAxis[i] set_acceleration:[sender intValue]];
    if (!err) err = [gAxis[i] get_acceleration];
  } else 
  if ((i=[edit_dec indexOfObject:sender]) != NSNotFound) {
    err = [gAxis[i] set_deceleration:[sender intValue]];
    if (!err) err = [gAxis[i] get_deceleration];
  } else {
    assert(0);
  }
  if (err) [self showError:err name:PREFUN];
  [self updateAxis:(int)i fromGalil:((err) ? YES : NO)];
}

/* ---------------------------------------------------------------- */

- (IBAction)action_stop:(id)sender
{
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);

  if (![self check_configured]) return;

  if ((i=[but_stop indexOfObject:sender]) != NSNotFound) {
    [gAxis[i] stop];
    [[slider_jog objectAtIndex:i] setDoubleValue:0];
  } else {                             // emergency STOP all axes
    int err = [gConnection open];
    if (!err) {
      [gConnection command:"AB"];
      Ysleep(500);
      for (i=0; i<GALIL_AXES; i++) {
        if (gAxis[i].motor_off) [gAxis[i] motorPower:NO];
        [self updateAxis:(int)i fromGalil:YES];
      }
      [gConnection close];
    }
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_jog:(id)sender
{
  NSInteger i;

  if (![self check_configured]) return;

  if ((i=[slider_jog indexOfObject:sender]) == NSNotFound) {
    assert(0);
    return;
  }
  assert(i < GALIL_AXES);  
  
  int j = (int)(gAxis[i].run_speed * [sender doubleValue]);
  if (j != jog_last) {                 // set new jog speed
    jog_last = j; [gAxis[i] jog:j];
    if (j) {
      if ([jog_lock tryLock]) {
        [self performSelectorInBackground:@selector(run_waitForJog:)
              withObject:[NSNumber numberWithInteger:i]];
      }
    }
  } else {                             // stop jog
    if ([NSEvent pressedMouseButtons] == 0) { // BigSur fix
      [gAxis[i] jog:0]; jog_last = 0;
      [sender setIntValue:jog_last];
    }
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_update:(id)sender
{
  int err;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  if (![self check_configured]) return;
  
  err = [gConnection open];
  if (!err) {
    for (int i=0; i<GALIL_AXES; i++) {
      if (!active[i]) continue;
      err = [self updateAxis:i fromGalil:YES]; if (err) break;
    }
    for (int i=0; i<GALIL_WHEELS; i++) {
      if (gWheel[i].nPositions == 0) continue;
      err = [self updateWheel:i fromGalil:YES]; if (err) break;
    }
    [gConnection close];
  }
  if (err) [self showError:err name:PREFUN];
}

/* ---------------------------------------------------------------- */

- (IBAction)action_terminal:(id)sender
{
  assert([NSThread isMainThread]);

  if (![self check_configured]) return;

  if (sender == edit_command) { int err; char cmd[128],res[GALIL_RESPONSE_SIZE];
    YeditGet(sender,cmd,sizeof(cmd));  // terminal command
    err = [gConnection request:cmd response:res max:sizeof(res)];
    if (err) {
      sprintf(res,"I/O failed, err=%d",err); [galil_logger message:res];
    }
#if 0 // print reason for '?'
    if (!err && strchr(res,'?')) {     // invalid command
      if (![gConnection errorMessage:res]) printf("%s\n",res);
    }
#endif
    if ([command_history count] > 0) { // check for same command
      if (strcmp(cmd,Ystr2chr([command_history objectAtIndex:0],res,sizeof(res)))) {
        [command_history insertObject:@(cmd) atIndex:0];
      }
    } else [command_history insertObject:@(cmd) atIndex:0];
    if ([command_history count] > [step_command maxValue]) {
      [command_history removeLastObject];
    }
    [step_command setDoubleValue:0];
  } else
  if (sender == step_command) {        // terminal history
    if ([command_history count]) {     // do we have a history ?
      if ([step_command intValue] >= [command_history count]) {
        [step_command setDoubleValue:([command_history count]-1)];
      }
      NSString *string = [command_history objectAtIndex:[step_command intValue]];
      [edit_command setStringValue:string];
    }
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_wheel:(id)sender
{
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif
  assert([NSThread isMainThread]);

  if (![self check_configured]) return;

  if ((i=[drop_wheel indexOfObject:sender]) == NSNotFound) {
    assert(0);
    return;
  }

  int w = (int)i;                      // wheel number {0..3}
  [self enable:NO wheel:w];
  GalilWheel *wheel = gWheel[w];
  int p = (int)[sender indexOfSelectedItem]-1;
  printf("w=%d --> %d,p\n",w,p);

  dispatch_async(GLOBAL_QUEUE,^(void) {
    int err = [wheel move:p];
    if (err) { char buf[128];
      sprintf(buf,"Wheel-%d failed, err=%d",w,err);
      [Notice openNotice:buf title:p_exec time:30 level:1];
    }
    [self updateWheel:w fromGalil:NO];
    [self updateAxisFromWheel:w];
    [app.camController updateWheel:w fromGalil:NO]; // b0077
    [self enable:YES wheel:w];
  });
}

/* ---------------------------------------------------------------- */

- (IBAction)action_home:(id)sender  
{
  NSInteger i;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  if (![self check_configured]) return;

  if ((i=[but_home indexOfObject:sender]) == NSNotFound) {
    assert(0);
    return;
  }

  int w = (int)i;                      // wheel number {0..3}
  [self enable:NO wheel:w];
  GalilWheel *wheel = gWheel[w];
  dispatch_async(GLOBAL_QUEUE,^(void) {
    int err = [gConnection open];
    if (!err) {
      [wheel performSelectorInBackground:@selector(find_home) withObject:nil];
      do {                             // do GUI updates
        Ysleep(350);
        [self updateWheel:w fromGalil:YES];
        Ysleep(350);
        [self updateAxisFromWheel:w];
      } while (wheel.homing);
      [self updateWheel:w fromGalil:YES];
      [self updateAxisFromWheel:w];
      [gConnection close];
      if ((wheel.state != 0) || (!wheel.foundHome)) err = E_galil_nohome;
    }
    [self enable:YES wheel:w];
    if (err) [self showError:err name:PREFUN];
    else [app.camController updateWheel:w fromGalil:NO]; 
  });
}

/* ---------------------------------------------------------------- */

#if 0 //TODO somewhere TESTING
  { char buf[128];
    sprintf(buf,"%c%c",1+'R'-'A',1+'S'-'A'); // ^R^S
    int err = [gConnection command:buf];
    printf("err=%d\n",err);
    if (!err) YprefPutChar(DBE_GALIL_ID,"unknown",use_shared);
    return;
  }
#endif

/* ---------------------------------------------------------------- */
#pragma mark "Update"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)updateGUI
{
  int i;
#if (DEBUG > 1)
  fprintf(stderr,"%s(): mainThread=%d\n",PREFUN,[NSThread isMainThread]);
#endif

  for (i=0; i<GALIL_AXES; i++) {
    if (!active[i]) continue;
    [self updateAxis:i fromGalil:NO];  // MT-safe
  }
  for (i=0; i<GALIL_WHEELS; i++) {
    if (gWheel[i].nPositions == 0) continue;
    [self updateWheel:i fromGalil:NO];
  }
}

/* ---------------------------------------------------------------- */

- (int)updateAxis:(int)a fromGalil:(BOOL)refresh
{
  int err=0;
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,a,refresh);
#endif
  assert((a>=0) && (a<GALIL_AXES));
  
  if (refresh) {
    err = [gConnection open]; 
    if (!err) {
      if (!err) err = [gAxis[a] get_pospar];
      if (!err) err = [gAxis[a] get_spdpar];
      if (!err) err = [gAxis[a] get_limits];
      [gConnection close];
    }
  }
  dispatch_async(MAIN_QUEUE,^(void) { // Note: these use an NSFormatter
    [[edit_stp objectAtIndex:a] setIntValue:gAxis[a].step_pos];
    [[edit_vel objectAtIndex:a] setIntValue:(int)my_round(gAxis[a].velocity,0)];
    assert([[edit_spd objectAtIndex:a] formatter]);
    assert([[edit_acc objectAtIndex:a] formatter]);
    assert([[edit_dec objectAtIndex:a] formatter]);
    [[edit_spd objectAtIndex:a] setIntValue:gAxis[a].speed];
    [[edit_acc objectAtIndex:a] setIntValue:gAxis[a].acc];
    [[edit_dec objectAtIndex:a] setIntValue:gAxis[a].dec];
    [[but_limf objectAtIndex:a] setState:(gAxis[a].limfor) ? NSControlStateValueOn : NSControlStateValueOff];
    [[but_limr objectAtIndex:a] setState:(gAxis[a].limrev) ? NSControlStateValueOn : NSControlStateValueOff];
  });
    
  return err;
}

/* ---------------------------------------------------------------- */

- (int)updatePosition:(int)a fromGalil:(BOOL)refresh
{
  int err=0;
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,a,refresh);
#endif
  assert((a>=0) && (a<GALIL_AXES));
  
  if (refresh) err = [gAxis[a] get_pospar];

  dispatch_async(MAIN_QUEUE,^(void) { // main thread
    assert([edit_stp[a] formatter]);
    assert([edit_vel[a] formatter]);
    [edit_stp[a] setIntValue:gAxis[a].step_pos];
    [edit_vel[a] setIntValue:(int)my_round(gAxis[a].velocity,0)];
  });

  return err;
}

/* ---------------------------------------------------------------- */

- (int)updateWheel:(int)w fromGalil:(BOOL)refresh
{
  int  err=0;
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,w,(int)refresh);
#endif
  assert((w>=0) && (w<=GALIL_WHEELS));

  if (!_configured) return 0;
  if (gWheel[w].nPositions == 0) return 0;

  if (refresh) err = [gWheel[w] get];

  if (!err) {
    dispatch_async(MAIN_QUEUE,^(void) { NSString *string;
      GalilWheel *wheel = gWheel[w];
      NSPopUpButton *drop = drop_wheel[w];
      if (wheel.state < 0) string = @"inv";
      else                 string = [drop itemTitleAtIndex:(1+wheel.state)];
      [drop setTitle:string];
      [but_dets[w] setState:((wheel.detent) ? NSControlStateValueOn : NSControlStateValueOff)];
      [but_homs[w] setState:((wheel.homesw) ? NSControlStateValueOn : NSControlStateValueOff)];
    });
  }

  return err;
}

/* ---------------------------------------------------------------- */

- (void)enable:(BOOL)e axis:(int)a
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,e,a);
#endif
  if (!active[a]) return;
  assert(active[a] || (e == NO));
    
  dispatch_async(MAIN_QUEUE,^(void) {
    [edit_rel[a] setEnabled:e];
    [but_relp[a] setEnabled:e];
    [but_relm[a] setEnabled:e];
    [slider_jog[a] setEnabled:e];
  });
}

/* ---------------------------------------------------------------- */

- (void)enable:(BOOL)e wheel:(int)w
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,e,w);
#endif

  dispatch_async(MAIN_QUEUE,^(void) {
    if (gWheel[w].nPositions) {
      [drop_wheel[w] setEnabled:e];
      [but_home[w] setEnabled:e];
    }
  });
}

/* ---------------------------------------------------------------- */
#pragma mark "Run"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)run_setup
{
  int    err,i;
  char   file[512],buf[512];
  Notice *note=nil;
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif
  _configured = FALSE;

  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

  if (gConnection.online) {
    note = [[Notice openNotice:"Connecting to Galil ..." title:p_exec] retain];
  }
#if (TIME_TEST > 0)
  NSDate *date = [NSDate date];
#endif

  err = [gConnection open];
  if (!err) {
    strcpy(file,"Galil Revision Number"); // request revision
    sprintf(buf,"%c%c",1+'R'-'A',1+'V'-'A'); // ^R^V
    err = [gConnection request:buf response:file];
    if (!err) [main_logger message:file file:YES];
    // Warning: none of the 'setup' happens if not connected
    sprintf(file,"%s/galil.xml",r_path);
    err = [gConnection setupFromFile:file];
    if (!err) {
      sprintf(file,"%s/galil",r_path);
      for (i=0; i<GALIL_AXES; i++) {   // setup all axes
        if (!active[i]) continue;
        err = [gAxis[i] setupFromFile:file]; if (err) break;
        dispatch_async(MAIN_QUEUE,^(void) { // GUI updates on mainThread
          [edit_rel[i] setStringValue:[NSString stringWithFormat:@"%d",gAxis[i].spt]];
          [but_clr[i]  setEnabled:YES];
        });
        [self enable:YES axis:i];
      } // endfor(axes)
    } // endif(err)
    if (!err) {
      for (i=0; i<GALIL_WHEELS; i++) {
        if (!wheels[i]) continue; 
        sprintf(file,"wheel%d",1+i);
        err = [gWheel[i] setup:file fromPath:r_path withAxis:gAxis[1+i]]; // b0075
        if (err) break;
        dispatch_async(MAIN_QUEUE,^(void) {
          [drop_wheel[i] setEnabled:YES]; [but_home[i] setEnabled:YES];
          for (int j=0; j<GALIL_AXES; j++) { // overwrite default steps
            if (gAxis[j].axis == gWheel[i].axis.axis) {
              [edit_rel[j] setStringValue:[NSString stringWithFormat:@"%d",gWheel[i].stepsPerPos]];
              break;
            }
          }
        });
      }
    }
    if (!err && gConnection.fileUploaded) {
      strcpy(file,"Galil Processor Serial Number");
      err = [gConnection request:"MG_BN" response:file];
      if (!err) {
        YprefGetChar(DBE_GALIL_ID,"?",buf,sizeof(buf),use_shared);
        if (strcmp(file,buf)) {        // unknown serial number
          YprefPutChar(DBE_GALIL_ID,file,use_shared);
#ifdef NDEBUG
          [gConnection command:"BN"];  // burn parameters to EEPROM -- TODO
          Ysleep(2000);
#endif
        }
        sprintf(buf,"Galil Processor ID: %s",file);
        [main_logger message:buf file:YES];
      }
    }
    [gConnection close];
  }
#if (TIME_TEST > 0)
  printf("%s: setupTime= %.3f\n",__FILE__,YsecSince(date));
#endif
  
  [note close]; [note release]; 
  
  if (err) {
    NSString *str = [NSString stringWithFormat:@"%s failed, err=%d\r%@",PREFUN,err,
                              [GalilHardhat errorString:err]];
    [Notice openNotice:[str UTF8String] title:p_exec time:0 level:2];
    for (i=0; i<GALIL_AXES; i++) [self enable:NO axis:i];
    for (i=0; i<GALIL_WHEELS; i++) [self enable:NO wheel:i];
  } else {
    assert(![NSThread isMainThread]);
    dispatch_sync(MAIN_QUEUE,^(void) {
      for (int w=0; w<GALIL_WHEELS; w++) {  
        int n = gWheel[w].nPositions; // printf("w=%d, n=%d (%p)\n",w,n,gWheel[w]);
        if (n) {
          for (int i=0; i<n; i++) {
            NSString *string = [NSString stringWithFormat:@"Pos%d",1+i];
            [drop_wheel[w] addItemWithTitle:string];
          }
        } else {
          [self enable:NO wheel:w];
        }
      }
    });
    _gAxes = [[NSArray alloc] initWithObjects:gAxis count:GALIL_AXES]; // b0075
    _gWheels = [[NSArray alloc] initWithObjects:gWheel count:GALIL_WHEELS]; // b0077
    _configured = TRUE;
  }
  
  [self updateGUI];
  
  [pool drain];
}

/* ---------------------------------------------------------------- */

- (void)run_check
{
  int  i,err=0;
  BOOL checking;                       // any check-mark
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif

  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

  @synchronized(gConnection) {         // b0458
    if (!check_open) {
      err = [gConnection open]; if (!err) check_open = TRUE;
    }
  }
  
  while (!err) {
    checking = FALSE;
    for (i=0; i<GALIL_AXES; i++) {
      if (check_states[i] == NSControlStateValueOn) {
        err = [self updatePosition:i fromGalil:YES];
        checking = TRUE; Ysleep(100);
      }
      if (err) break;
      Ysleep(125);
    }
    if (!checking) break;
  }
  if (err) {
    NSString *s = [NSString stringWithFormat:@"%s(): err=%d\r%@",
                    PREFUN,err,[[self class] errorString:err]];
    [main_logger message:[s UTF8String] level:1];
  }
  
  @synchronized(gConnection) {
    if (check_open) {
      [gConnection close]; check_open = FALSE;
    }
  }
#if (DEBUG > 0)
  fprintf(stderr,"%s() ... done\n",PREFUN);
#endif
  [pool drain];
}

/* ---------------------------------------------------------------- */

- (void)run_waitForAxis:(NSNumber*)axis
{
  int  err,a=[axis intValue];
  BOOL running,limits=FALSE;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d)\n",PREFUN,a);
#endif

  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

  if (gAxis[a].limfor || gAxis[a].limrev) limits = TRUE; 
  
  err = [gConnection open]; 
  if (!err) {                          // wait for axis to stop
    do {
      Ysleep(1000);                    // once per second
      running = gAxis[a].running;
      if (limits) [self updateAxis:a fromGalil:YES];
      else        [self updatePosition:a fromGalil:YES];
    } while (running);
    do {                               // wait for velocity=0
      Ysleep(350);
      err = [gAxis[a] get_velocity]; if (err) break;
    } while (gAxis[a].velocity != 0);
    Ysleep(500);
    [self updatePosition:a fromGalil:YES];
    [gConnection close];
  }

  [pool drain];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d) ... done\n",PREFUN,a);
#endif
}

/* ---------------------------------------------------------------- */

- (void)run_waitForJog:(NSNumber*)axis
{
  int err,a=[axis intValue];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d)\n",PREFUN,a);
#endif

  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

  err = [gConnection open];
  if (!err) {
    do {
      Ysleep(500);                     // twice per second
      [gAxis[a] isJogging];
      err = [gAxis[a] get_step_pos]; if (err) break;
      err = [gAxis[a] get_speed]; if (err) break;
      err = [gAxis[a] get_velocity]; if (err) break;
      err = [gAxis[a] get_limits]; if (err) break; 
      [self updateAxis:a fromGalil:NO];
    } while (gAxis[a].jogging);
    do {
      Ysleep(350);                    // wait for velocity=0
      err = [gAxis[a] get_velocity]; if (err) break;
    } while (gAxis[a].velocity != 0);
    [self updateAxis:a fromGalil:YES];
    [gConnection close];
  }

  [jog_lock performSelectorOnMainThread:@selector(unlock)
            withObject:nil waitUntilDone:YES];

  [pool drain];  
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d) ... done\n",PREFUN,a);
#endif
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
