/* ---------------------------------------------------------------- */
//
// GraphController.h
// FourStar / Henrietta
//
// Created by Christoph Birk on 2014-09-10
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

/* ---------------------------------------------------------------- */

typedef struct {
  float r,g,b;
} GraphColor;

/* ---------------------------------------------------------------- */

@interface GraphDataPoint : NSObject {
  time_t time;
  float  value;
}
@property (readonly,nonatomic) time_t time;
@property (readonly,nonatomic) float  value;
+ (id)pointWithTime:(time_t)t value:(float)v; 
- (id)initWithTime:(time_t)t value:(float)v;
@end

/* ---------------------------------------------------------------- */

enum graph_scale_enum { GRAPH_SCALE_MAN,GRAPH_SCALE_FULL,GRAPH_SCALE_LAST };

@interface GraphView : NSView {
  NSArray    *dataArrays;
  BOOL       *monitor;
  GraphColor *colors;
  NSString   *unit;
  int        scale_tmode,scale_vmode;
  time_t     scale_tmin,scale_tmax,scale_tlst;
  float      scale_vmin,scale_vmax;
  CGFloat    font_size;
  CTFontRef  font;
}

@property (assign,nonatomic) int      scale_tmode,scale_vmode;
@property (assign,nonatomic) time_t   scale_tmin,scale_tmax,scale_tlst;
@property (assign,nonatomic) float    scale_vmin,scale_vmax;
@property (copy,nonatomic)   NSString *unit;

- (void)setDataArray:(NSArray*)array colors:(const GraphColor*)cols;
- (void)setChannel:(int)c monitor:(BOOL)m;

@end

/* ---------------------------------------------------------------- */

@interface GraphController : NSWindowController {
           time_t        time0;
           NSString      *key_frame;
           NSArray       *dataArrays;
  IBOutlet GraphView     *view_graph;
  IBOutlet NSPanel       *panel_gpars;
  IBOutlet NSPopUpButton *popup_tmode,*popup_vmode;
  IBOutlet NSTextField   *edit_tmin,*edit_tmax,*edit_vmin,*edit_vmax;
  IBOutlet NSButton      *but_apply;
}

@property (readonly,nonatomic) GraphView *view_graph;

- (id)initWithWindowNib:(NSString*)name context:(int)myid
                                        title:(const char*)tstr
                                        unit:(const char*)ustr
                                        N:(int)nc
                                        colors:(const GraphColor*)cols;

- (IBAction)action_gpars:(id)sender;

- (void)addData:(int)c time:(time_t)t value:(float)v redraw:(BOOL)r;
- (void)addData:(int)c time:(time_t)t value:(float)v;
- (void)addData:(int)c value:(float)v redraw:(BOOL)r;
- (void)addData:(int)c value:(float)v;
- (void)thinData:(double)threshold after:(double)maxage;
- (void)setChannel:(int)c monitor:(BOOL)m;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
