/* ---------------------------------------------------------------- */
//
// QltoolController.h
// Swope/PFS
//
// Created by Christoph Birk on 2011-12-08
// Copyright 2011 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import  "OverView.h"
#import  "MagView.h"
#import  "MTwister.h"

/* ---------------------------------------------------------------- */

#define QLT_NCOLS       200

enum { LUT_GRAY,LUT_BBDY,LUT_RAIN,LUT_INV };

#define LUT_GREY LUT_GRAY

enum { STRETCH_CUTS,STRETCH_MED3,STRETCH_MED5,STRETCH_310,STRETCH_MIMA };

#define DBE_SATLEV      "dbe_satlev"
#define DEF_SATLEV      60000   // ?Nick

/* ---------------------------------------------------------------- */

@interface QltoolController : NSWindowController {
  // GUI
  int      lut,stretch,satlev,radius; 
  double   slevel,contrast;
  u_int    *pixels;
  MTwister *twister;
  // Overview
  IBOutlet OverView      *over_view;
  IBOutlet NSTextField   *edit_min,*edit_max;
  IBOutlet NSPopUpButton *popup_lut,*popup_stretch;
  IBOutlet NSTextField   *edit_frame;
  IBOutlet NSStepper     *step_frame;
  // Magnifier
  IBOutlet MagView *mag_view;       
  IBOutlet NSTextField *edit_pixel,*edit_minmax,*edit_avesig;
  IBOutlet NSTextField *edit_radius,*edit_fwhm;
  IBOutlet NSStepper *step_radius;
  IBOutlet NSPopUpButton *popup_mag;
  int      mag,mag_x,mag_y;
  float    fwhm,flux;
}

@property (readonly)           u_int  raw_w,raw_h;

@property (nonatomic)          double contrast,slevel;
@property (nonatomic)          float  fwhm;
@property (readonly,nonatomic) u_int  *pixels;
@property (readonly,nonatomic) int    satlev,lut;

@property (assign,nonatomic,readonly) MagView *mag_view;
@property (readonly)        int mag,mag_x,mag_y;

- (void)reloadPreferences:(BOOL)redraw;
- (void)fill_pixels;

- (IBAction)action_lut:(id)sender;
- (IBAction)action_stretch:(id)sender;
- (IBAction)action_minmax:(id)sender;
- (IBAction)action_mag:(id)sender;
- (IBAction)action_radius:(id)sender;
- (IBAction)action_sky:(id)sender;
- (IBAction)action_frame:(id)sender;

- (void)resetTitle:(NSString*)tstr;
- (void)copyData:(int*)data asSky:(BOOL)sky;
- (void)addFrame:(NSData*)data;
- (void)addRawFrame:(NSData*)data;
- (void)redrawWithScaling:(BOOL)scale;
- (void)redrawPixels;
- (void)update_magnifierX:(float)mx Y:(float)my;
- (void)update_fwhm:(BOOL)recalc;

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
