/* ----------------------------------------------------------------
 *
 * utils.h
 * 
 * Project:  TeleGUI/Gmap  (Carnegie Observatories, Pasadena, CA)
 *
 * Christoph C. Birk (birk@obs.carnegiescience.edu)
 *
 * ---------------------------------------------------------------- */

#ifndef INCLUDE_UTILS_H
#define INCLUDE_UTILS_H

/* ---------------------------------------------------------------- */

#include <time.h>                      // time_t

/* GLOBALs -------------------------------------------------------- */

#ifndef RADS
#define RADS          (180.0/M_PI)
#endif

/* ---------------------------------------------------------------- */

double get_sidereal   (time_t,double);
double get_st         (time_t,double);
double get_airmass    (double,double,double);
double get_coszd      (double lat,double dec,double ha);
double get_ha         (time_t,double,double);
double get_para       (double ha,double dec,double lat);
void   get_altaz      (double st,double lat,double ra,double dec,double* alt,double* az);
void   get_radec      (double st,double lat,double alt,double az,double* ra,double* dec);
double get_epoch      (time_t);
char*  get_night      (char*,time_t);
int    get_uts        (time_t);
time_t cor_time       (long);
double walltime       (void);

char*  time_str       (char*,double);
char*  alpha_str      (char*,double);
char*  delta_str      (char*,double);
double d_alpha        (const char*);
double d_delta        (const char*);
double d_degrees      (const char*);
double d_equinox      (const char*);

void   precess        (double,double,double,double*,double*,double);

void   q_sort         (int*,int,int);
void   f_sort         (float*,int,int);
void   u_sort         (u_short* zahl, int l, int r);

char*  my_extend      (char*,char,int);
char*  cut_spaces     (char*);
char*  trim_string    (char*,unsigned int);
char* lowercase       (char *string);
char* uppercase       (char *string);

int    imin               (int,int);
int    imax               (int,int);
long   lmax               (long a,long b);
long   lmin               (long a,long b);

char*  get_local_datestr  (char*,time_t);
char*  get_local_timestr  (char*,time_t);
char*  get_ut_datestr     (char* string,time_t ut);
char*  get_ut_timestr     (char*,time_t);

double mag2flux  (double,double);
double flux2peak (double,double,double);
double peak2flux (double,double);
double flux2mag  (double,double);
double mag2fluxC (double v,double r,double cfzm,double airmass);
double flux2magC (double flux,double v_r,double cfzm,double airmass);

int    is_bigendian       (void);
double my_round           (double,int);

double modulo             (double,double,double);
double modneg             (double,double);
double modtop             (double,double);
double sqd                (double);

/* ---------------------------------------------------------------- */

#endif  // INCLUDE_UTILS_H

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
